@@ -151,6 +151,12 @@ class DatarateTestLarge
                               "pixel_capture_w320h240.yuv", 100);
     const int bitrate_array[2] = { 1000, 2000 };
     cfg_.rc_target_bitrate = bitrate_array[GET_PARAM(4)];
+#ifdef AOM_VALGRIND_BUILD
+    if (cfg_.rc_target_bitrate == 2000) {
+      GTEST_SKIP() << "No need to run this test for 2 bitrates, the issue for "
+                      "this test occurs at first bitrate = 1000.";
+    }
+#endif  // AOM_VALGRIND_BUILD
     ResetModel();
     avif_mode_ = 1;
     ASSERT_NO_FATAL_FAILURE(RunLoop(&video));
@@ -759,6 +765,59 @@ TEST_P(DatarateTestSetFrameQpRealtime, SetFrameQpOnePass) {
   ASSERT_NO_FATAL_FAILURE(RunLoop(&video));
 }
 
+class DatarateTestPsnr
+    : public DatarateTest,
+      public ::testing::TestWithParam<const libaom_test::AV1CodecFactory *> {
+ public:
+  DatarateTestPsnr() : DatarateTest(GetParam()) {}
+
+ protected:
+  ~DatarateTestPsnr() override = default;
+
+  void SetUp() override {
+    InitializeConfig(libaom_test::kRealTime);
+    ResetModel();
+    set_cpu_used_ = 10;
+    frame_flags_ = AOM_EFLAG_CALCULATE_PSNR;
+    expect_psnr_ = true;
+  }
+  void PreEncodeFrameHook(::libaom_test::VideoSource *video,
+                          ::libaom_test::Encoder *encoder) override {
+    DatarateTest::PreEncodeFrameHook(video, encoder);
+    frame_flags_ ^= AOM_EFLAG_CALCULATE_PSNR;
+#if CONFIG_INTERNAL_STATS
+    // CONFIG_INTERNAL_STATS unconditionally generates PSNR.
+    expect_psnr_ = true;
+#else
+    expect_psnr_ = (frame_flags_ & AOM_EFLAG_CALCULATE_PSNR) != 0;
+#endif  // CONFIG_INTERNAL_STATS
+    if (video->img() == nullptr) {
+      expect_psnr_ = false;
+    }
+  }
+  void PostEncodeFrameHook(::libaom_test::Encoder *encoder) override {
+    libaom_test::CxDataIterator iter = encoder->GetCxData();
+
+    bool had_psnr = false;
+    while (const aom_codec_cx_pkt_t *pkt = iter.Next()) {
+      if (pkt->kind == AOM_CODEC_PSNR_PKT) had_psnr = true;
+    }
+
+    EXPECT_EQ(had_psnr, expect_psnr_);
+  }
+
+ private:
+  bool expect_psnr_;
+};
+
+TEST_P(DatarateTestPsnr, PerFramePsnr) {
+  ::libaom_test::I420VideoSource video("hantro_collage_w352h288.yuv", 352, 288,
+                                       30, 1, 0, 100);
+
+  ResetModel();
+  ASSERT_NO_FATAL_FAILURE(RunLoop(&video));
+}
+
 AV1_INSTANTIATE_TEST_SUITE(DatarateTestLarge,
                            ::testing::Values(::libaom_test::kRealTime),
                            ::testing::Range(5, 7), ::testing::Values(0, 3),
@@ -786,5 +845,10 @@ INSTANTIATE_TEST_SUITE_P(
     ::testing::Values(
         static_cast<const libaom_test::CodecFactory *>(&libaom_test::kAV1)));
 
+INSTANTIATE_TEST_SUITE_P(
+    AV1, DatarateTestPsnr,
+    ::testing::Values(
+        static_cast<const libaom_test::CodecFactory *>(&libaom_test::kAV1)));
+
 }  // namespace
 }  // namespace datarate_test
